no warnings qw(redefine);

sub CreateTicket {
    my %ARGS = (@_);

    my (@Actions);

    my $Ticket = new RT::Ticket( $session{'CurrentUser'} );

    my $Queue = new RT::Queue( $session{'CurrentUser'} );
    unless ( $Queue->Load( $ARGS{'Queue'} ) ) {
        Abort('Queue not found');
    }

    unless ( $Queue->CurrentUserHasRight('CreateTicket') ) {
        Abort('You have no permission to create tickets in that queue.');
    }

    my $due = new RT::Date( $session{'CurrentUser'} );
    $due->Set( Format => 'unknown', Value => $ARGS{'Due'} );
    my $starts = new RT::Date( $session{'CurrentUser'} );
    $starts->Set( Format => 'unknown', Value => $ARGS{'Starts'} );

    my @Requestors = split ( /\s*,\s*/, $ARGS{'Requestors'} );
    my @Cc         = split ( /\s*,\s*/, $ARGS{'Cc'} );
    my @AdminCc    = split ( /\s*,\s*/, $ARGS{'AdminCc'} );

    my $MIMEObj = MakeMIMEEntity(
        Subject             => $ARGS{'Subject'},
        From                => $ARGS{'From'},
        Cc                  => $ARGS{'Cc'},
        Body                => $ARGS{'Content'},
    );

    if ( $ARGS{'Attachments'} ) {
        my $rv = $MIMEObj->make_multipart;
        $RT::Logger->error("Couldn't make multipart message")
            if !$rv || $rv !~ /^(?:DONE|ALREADY)$/;

        foreach ( values %{$ARGS{'Attachments'}} ) {
            unless ( $_ ) {
                $RT::Logger->error("Couldn't add empty attachemnt");
                next;
            }
            $MIMEObj->add_part($_);
        }
    }

    my %create_args = (
        Type            => $ARGS{'Type'} || 'ticket',
        Queue           => $ARGS{'Queue'},
        Owner           => $ARGS{'Owner'},
        InitialPriority => $ARGS{'InitialPriority'},
        FinalPriority   => $ARGS{'FinalPriority'},
        TimeLeft        => $ARGS{'TimeLeft'},
        TimeEstimated        => $ARGS{'TimeEstimated'},
        TimeWorked      => $ARGS{'TimeWorked'},
        Subject         => $ARGS{'Subject'},
        Status          => $ARGS{'Status'},
        Due             => $due->ISO,
        Starts          => $starts->ISO,
        MIMEObj         => $MIMEObj
    );
    
    my @temp_squelch;
    foreach my $type (qw(Requestors Cc AdminCc)) {
        my @tmp = map { $_->format } grep { $_->address} Mail::Address->parse( $ARGS{ $type } );

        $create_args{ $type } = [
            grep $_, map {
                my $user = RT::User->new( $RT::SystemUser );
                $user->LoadOrCreateByEmail( $_ );
                # convert to ids to avoid work later
                $user->id;
            } @tmp
        ];
        $RT::Logger->debug(
            "$type got ".join(',',@{$create_args{ $type }}) );

    }
    # XXX: workaround for name conflict :(
    $create_args{'Requestor'} = delete $create_args{'Requestors'};
        
    foreach my $arg (keys %ARGS) {
        next if $arg =~ /-(?:Magic|Category)$/;

        if ($arg =~ /^Object-RT::Transaction--CustomField-/) {
            $create_args{$arg} = $ARGS{$arg};
        }
        # Object-RT::Ticket--CustomField-3-Values
        elsif ($arg =~ /^Object-RT::Ticket--CustomField-(\d+)(.*?)$/) {
            my $cfid = $1;
            my $cf = RT::CustomField->new( $session{'CurrentUser'});
            $cf->Load($cfid);

            if ( $cf->Type eq 'Freeform' && ! $cf->SingleValue) {
                $ARGS{$arg} =~ s/\r\n/\n/g;
                $ARGS{$arg} = [split('\n', $ARGS{$arg})];
            }

            if ( $cf->Type =~ /text/i) { # Catch both Text and Wikitext
                $ARGS{$arg} =~ s/\r//g;
            }

            if ( $arg =~ /-Upload$/ ) {
                $create_args{"CustomField-".$cfid} = _UploadedFile($arg);
            }
            else {
                $create_args{"CustomField-".$cfid} = $ARGS{"$arg"};
            }
        }
    }


    # XXX TODO This code should be about six lines. and badly needs refactoring.

    # {{{ turn new link lists into arrays, and pass in the proper arguments
    my (@dependson, @dependedonby, @parents, @children, @refersto, @referredtoby);

    foreach my $luri ( split ( / /, $ARGS{"new-DependsOn"} ) ) {
        $luri =~ s/\s*$//;    # Strip trailing whitespace
        push @dependson, $luri;
    }
    $create_args{'DependsOn'} = \@dependson;

    foreach my $luri ( split ( / /, $ARGS{"DependsOn-new"} ) ) {
        push @dependedonby, $luri;
    }
    $create_args{'DependedOnBy'} = \@dependedonby;

    foreach my $luri ( split ( / /, $ARGS{"new-MemberOf"} ) ) {
        $luri =~ s/\s*$//;    # Strip trailing whitespace
        push @parents, $luri;
    }
    $create_args{'Parents'} = \@parents;

    foreach my $luri ( split ( / /, $ARGS{"MemberOf-new"} ) ) {
        push @children, $luri;
    }
    $create_args{'Children'} = \@children;

    foreach my $luri ( split ( / /, $ARGS{"new-RefersTo"} ) ) {
        $luri =~ s/\s*$//;    # Strip trailing whitespace
        push @refersto, $luri;
    }
    $create_args{'RefersTo'} = \@refersto;

    foreach my $luri ( split ( / /, $ARGS{"RefersTo-new"} ) ) {
        push @referredtoby, $luri;
    }
    $create_args{'ReferredToBy'} = \@referredtoby;
    # }}}
   
    my ( $id, $Trans, $ErrMsg ) = $Ticket->Create(%create_args);
    unless ( $id ) {
        return ($id, $ErrMsg);
    }

    push ( @Actions, split("\n", $ErrMsg) );
    unless ( $Ticket->CurrentUserHasRight('ShowTicket') ) {
        Abort( "No permission to view newly created ticket #"
            . $Ticket->id . "." );
    }
    return ( $Ticket, @Actions );

}

1;
